#include <avr/io.h>
#include <stdlib.h>
#define F_CPU 16000000
#include <util/delay.h>

#define MORSE_LUNG 3
#define MORSE_SCURT 1


int pini_linie[8] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};
int pini_coloana[8] = {PD0, PD1, PD2, PD3, PD4, PD5, PD6, PD7};


int game_map[8][8]; 
int red_start = 3;
int red_left = 0, red_right = 0;
int green_start = 3;
int green_left = 0, green_right = 0;

int ball_direction = 0;
int ball_x, ball_y;
int srnd = 0;

int goal_scored = 0;
int red_goals = 0;
int green_goals = 0;

/* Init registers */
void init_ports(){

	/* Red player buttons */
	DDRB &= ~(1 << PB0);
	PORTB |= (1 << PB0);
	
	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);
	
	/* Red player LEDs */
	DDRB |= (1 << PB2);
	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);
	PORTB &= ~(1 << PB2);
	PORTB &= ~(1 << PB3);
	PORTB &= ~(1 << PB4);
	
	/* Green player buttons */	
	DDRC &= ~(1 << PC1);  
	PORTC |= (1 << PC1);  
	DDRC &= ~(1 << PC2);  
	PORTC |= (1 << PC2);  
	
	/* Green player LEDs */
	DDRC |= (1 << PC3);
	DDRC |= (1 << PC4);
	DDRC |= (1 << PC5);
	PORTC &= ~(1 << PC3);
	PORTC &= ~(1 << PC4);
	PORTC &= ~(1 << PC5);

	/* Init matrix ports */
	/* Lines */
	DDRA |= (1 << pini_linie[0]);
	DDRA |= (1 << pini_linie[1]);
	DDRA |= (1 << pini_linie[2]);
	DDRA |= (1 << pini_linie[3]);
	DDRA |= (1 << pini_linie[4]);
	DDRA |= (1 << pini_linie[5]);
	DDRA |= (1 << pini_linie[6]);
	DDRA |= (1 << pini_linie[7]);
	
	/* Columns*/
	DDRD |= (1 << pini_coloana[0]);
	DDRD |= (1 << pini_coloana[1]);
	DDRD |= (1 << pini_coloana[2]);
	DDRD |= (1 << pini_coloana[3]);
	DDRD |= (1 << pini_coloana[4]);
	DDRD |= (1 << pini_coloana[5]);
	DDRD |= (1 << pini_coloana[6]);
	DDRD |= (1 << pini_coloana[7]);
}




/* Init game matrix map */
void init_map() {
	int i,j;
	for (i = 0; i < 8; i++){
		for (j = 0; j < 8; j++){
			game_map[i][j] = 0;
		}
		
	}
	
	/* Green player */
	green_start = 3;
	game_map[0][green_start] = 1;
	game_map[0][green_start + 1] = 1;
	
	/* Red player */
	red_start = 3;
	game_map[7][red_start] = 1;
	game_map[7][red_start + 1] = 1;
	
	/* Ball */
	srand(srnd++);
	int ball_side = rand() % 4;
	if (ball_side == 0){
		game_map[3][3] = 2;	
		ball_x = 3;
		ball_y = 3;
	}
	if (ball_side == 1){
		game_map[4][3] = 2;	
		ball_x = 4;
		ball_y = 3;
	}
	if (ball_side == 2){
		game_map[3][4] = 2;	
		ball_x = 3;
		ball_y = 4;
	}
	if (ball_side == 3){
		game_map[4][4] = 2;	
		ball_x = 4;
		ball_y = 4;
	}
	
	/* Ball direction */
	srand(srnd++);
	ball_direction = rand() % 6;
}

void draw_map(){
	int i,j;
	for (i = 0;i < 8; i++){
		for (j = 0; j < 8 ; j++){
			if (game_map[i][j] > 0){
				PORTD |= (1 << pini_coloana[j]);
			}
			else{
				PORTD &= ~(1 << pini_coloana[j]);
			}
		}
		PORTA |= (1 << pini_linie[i]);
		_delay_ms(0.5);
		PORTA &= ~(1 << pini_linie[i]);
	}

}

/* Update the positions of the players and of the ball */
void update_map(){
	int i,j;
	
	/* Clear map */
	for (i = 0; i < 8; i++){
		for (j = 0; j < 8; j++){
			game_map[i][j] = 0;
		}
	}
	
	/* Write players */
	game_map[0][green_start] = 1;
	game_map[0][green_start + 1] = 1;
	game_map[7][red_start] = 1;
	game_map[7][red_start + 1] = 1;
	
	/* Write ball */
	game_map[ball_x][ball_y] = 2;	
}

/* Checks if the buttons were pressed */
void run_listeners(){
	/* Red player buttons */
	if (!(PINB & (1 << PB1))){
		if (red_start != 6 && red_right == 0){
			red_start++;
			red_right = 1;
			update_map();
		}
	}
	else{
		red_right = 0;
	}
		
	if (!(PINB & (1 << PB0))){
		if (red_start != 0 && red_left == 0){
			red_start--;
			red_left = 1;
			update_map();
		}
	}
	else{
		red_left = 0;
	}
	
	/* Green player buttons */
	if (!(PINC & (1 << PC2))){
		if (green_start != 6 && green_right == 0){
			green_start++;
			green_right = 1;
			update_map();
		}
	}
	else{
		green_right = 0;
	}
		
	if (!(PINC & (1 << PC1))){
		if (green_start != 0 && green_left == 0){
			green_start--;
			green_left = 1;
			update_map();
		}
	}
	else{
		green_left = 0;
	}
	
}


/* Updates the position of the ball */
void update_ball(){
	
	int rerun_update = 0;
	/* Update position */

	if (ball_direction == 0){
		if (ball_x > 0 && ball_y > 0){
			ball_x--;
			ball_y--;
			goto map_update;
		}
		else{
			if (ball_x == 0) {
				goal_scored = 1;
				return;
			}
			else{
				ball_direction = 2;
				rerun_update = 1;
				goto player_collision;
			}
		}
		
	}
	
	if (ball_direction == 1){
		if (ball_x > 0){
			ball_x--;
			goto map_update;
		}
		else{
			goal_scored = 1;
			return;
		}
		
	}
	
	if (ball_direction == 2){
		if (ball_x > 0 && ball_y < 7){
			ball_x--;
			ball_y++;
			goto map_update;
		}
		else{
			if (ball_x == 0){
				goal_scored = 1;
				return;
			}
			else{
				ball_direction = 0;
				rerun_update = 1;
				goto player_collision;
			}
			
		}
		
	}
	
	if (ball_direction == 5){
		if (ball_x < 7 && ball_y < 7){
			ball_x++;
			ball_y++;
			goto map_update;
		}
		else{
			if (ball_x == 7){
				goal_scored = 1;
				return;
			}
			else{
				ball_direction = 3;
				rerun_update = 1;
				goto player_collision;
			}
		}
		
	}
	
	if (ball_direction == 4){
		if (ball_x < 7){
			ball_x++;
			goto map_update;
		}
		else{
			goal_scored = 1;
			return;
		}
		
	}
	
	if (ball_direction == 3){
		if (ball_x < 7 && ball_y > 0){
			ball_x++;
			ball_y--;
			goto map_update;
		}
		else{
			if (ball_x == 7){
				goal_scored = 1;
				return;
			}
			else{
				ball_direction = 5;
				rerun_update = 1;
				goto player_collision;
			}
		}
		
	}
	
map_update:
	update_map();

player_collision:
	if (ball_direction == 0 && ball_x - 1 == 0){
		if (ball_y - 1  == green_start){
			ball_direction = 3 + (rand() % 3);
		} 
		if (ball_y - 1 == green_start + 1){
			ball_direction = 5;
		}
	}
	
	if (ball_direction == 1 && ball_x - 1 == 0){
		if (ball_y == green_start || ball_y == green_start + 1){
			ball_direction = 3 + (rand() % 3);
		}
	}
	
	if (ball_direction == 2 && ball_x - 1 == 0){
		
		if (ball_y + 1  == green_start){
			ball_direction = 3;
			
		} 
		if (ball_y + 1 == green_start + 1){
			ball_direction = 3 + (rand() % 3);
		}
	}
	
	
	if (ball_direction == 3 && ball_x + 1 == 7){
		if (ball_y - 1  == red_start + 1){
			ball_direction = 2;
		}
		if (ball_y - 1 == red_start){
			ball_direction = 0 + (rand() % 3);
		}
	}
	
	if (ball_direction == 4 && ball_x + 1 == 7){
		if (ball_y == red_start || ball_y == red_start + 1){
			ball_direction = 0 + (rand() % 3);
		}
	}
	
	if (ball_direction == 5 && ball_x + 1 == 7){
		if (ball_y + 1 == red_start){
			ball_direction = 0;
		}
		if (ball_y + 1 == red_start + 1){
			ball_direction = 0 + (rand() % 3);
		}
	}
	
	if (rerun_update){
		update_ball();
	}
}


/* The functions that runs the game */
void run_game(){
	init_map();
	int step = 0;
	while(1){
		step++;
		if (step == 100){
			update_ball();
			if (goal_scored){
				if (ball_x == 0){
					if (red_goals == 0){
						PORTB |= (1 << PB2); 
					}
					
					if (red_goals == 1){
						PORTB |= (1 << PB3); 
					}
					
					if (red_goals == 2){
						PORTB |= (1 << PB4); 
					}
					
					red_goals++;
					
					goal_scored = 0;
					return;
				}
				
				if (ball_x == 7){
					if (green_goals == 0){
						PORTC |= (1 << PC3); 
					}
					
					if (green_goals == 1){
						PORTC |= (1 << PC4); 
					}
					
					if (green_goals == 2){
						PORTC |= (1 << PC5); 
					}
					
					green_goals++;
					goal_scored = 0;
					return;
				}
			}
			step = 0;
		}
		run_listeners();
		draw_map();
	}
}

/* Main function will run an instance of the game until a player scores 3 goals */
int main(void) {
	init_ports();
	while(green_goals != 3 && red_goals != 3){
		run_game();
	}
	return 0;
}

